#ifndef H_TYPETREE
#define H_TYPETREE

#include <cstdint>
#include <vector>

class TypeTree {
public:
    struct Node {
        uint32_t value;
        size_t   parent;
        std::vector<size_t> children;
        
        Node(uint32_t value, size_t parent);
    };

    TypeTree();
    ~TypeTree();

    bool has(size_t id);

    size_t outer(size_t id, uint32_t type);
    uint32_t value(size_t id);
    size_t inner(size_t id);

private:
    std::vector<Node> m_nodes;
};

TypeTree::Node::Node(uint32_t value, size_t parent) :value(value), parent(parent) { }

TypeTree::TypeTree()
{
    m_nodes.push_back(Node(0, 0));
}

TypeTree::~TypeTree()
{ }

bool TypeTree::has(size_t id)
{
    return id < m_nodes.size();
}

size_t TypeTree::outer(size_t id, uint32_t value)
{
    size_t find = 0;

    if(id < m_nodes.size())
    {
        Node& node = m_nodes[id];

        for(size_t child : node.children) {
            if(m_nodes[child].value == value) {
                find = child;
                break;
            }
        }

        if(find == 0) {
            m_nodes.push_back(Node(value, id));
            find = m_nodes.size() - 1;

            node.children.push_back(find);
        }
    }

    return find;
}

uint32_t TypeTree::value(size_t id)
{
    if(id > 0 && id < m_nodes.size()) {
        return m_nodes[id].value;
    }
    return 0;
}

size_t TypeTree::inner(size_t id)
{
    if(id > 0 && id < m_nodes.size()) {
        return m_nodes[id].parent;
    }
    return 0;
}

#endif
